#pragma semicolon 1

#include <sourcemod>
#include <adjuka>


#undef REQUIRE_EXTENSIONS
#include <tf2_stocks>
#include <steamtools>
#include <SteamWorks>

#pragma newdecls required

#define PLUGIN_VERSION "1.1.8"

// Forward Handles
GlobalForward g_Forward_OnAdDisplayed = null;

// Player Properties
int g_iDeaths[MAXPLAYERS + 1];
Handle g_hPeriodicTimer[MAXPLAYERS + 1];
Handle g_hSpectatorTimer[MAXPLAYERS + 1];
bool g_bFirstJoin[MAXPLAYERS + 1];
int g_iTimeLastAd[MAXPLAYERS + 1];
bool g_bHasMOTDEnabled[MAXPLAYERS + 1];

// Server Address
int g_iOctets[4];
char g_sHostIP[16];
int g_iHostPort;

// Game
EngineVersion g_EngineVersion;
char g_sGameDir[16];

// Required Extensions
bool g_bSteamTools;
bool g_bSteamWorks;

// ConVars
ConVar g_ConVar_CommunityID;
ConVar g_ConVar_ServerIP;
ConVar g_ConVar_XAmountDeaths;
ConVar g_ConVar_XAmountDeathMinutesEnabled;
ConVar g_ConVar_XAmountDeathMinutes;
ConVar g_ConVar_PeriodAmountMinutes;
ConVar g_ConVar_SpectatorAmountMinutes;
ConVar g_ConVar_OnPhase;
ConVar g_ConVar_ExcludeAdmins;
ConVar g_ConVar_ExcludeDonators;
ConVar g_ConVar_KickNoMOTDEnabled;

public Plugin myinfo =
{
	name = "Adjuka MOTD advertisement",
	author = "Adjuka",
	description = "Display display/rich media and video ads in the MOTD window via Adjuka",
	version = PLUGIN_VERSION,
	url = "https://adjuka.com/"
}

/* OnPluginStart()
** -------------------------------------------------------------------------- */
public void OnPluginStart()
{
	//Hook Messages
	UserMsg umVGUIMenu = GetUserMessageId("VGUIMenu");

	if (umVGUIMenu == INVALID_MESSAGE_ID)
	{
		SetFailState("The server's engine version doesn't supports VGUI menus.");
	}

	HookUserMessage(umVGUIMenu, Event_PlayerVGUIMenu, true);

	// Forwards
	g_Forward_OnAdDisplayed = new GlobalForward("Adjuka_OnAdDisplayed", ET_Event, Param_Cell);

	// ConVars
	CreateConVar("sm_adjuka_community_version", PLUGIN_VERSION, "Version ConVar for tracking", FCVAR_DONTRECORD);
	g_ConVar_CommunityID = CreateConVar("sm_adjuka_community_id", "", "Your community ID to identify ads being displayed by your community", _);
	g_ConVar_ServerIP = CreateConVar("sm_adjuka_server_ip", "", "Your server IP if the automatic detections do not work. (Example: 127.0.0.1:27015)");
	g_ConVar_XAmountDeaths = CreateConVar("sm_adjuka_every_x_deaths", "10", "Play an advert every time somebody dies this many times, 0 = Disabled.", _, true, 0.0);
	g_ConVar_XAmountDeathMinutesEnabled = CreateConVar("sm_adjuka_every_x_deaths_minutes_enabled", "1", "Should ad be played after death after certain amount of time be enabled, 0 = Disabled.", _, true, 0.0, true, 1.0);
	g_ConVar_XAmountDeathMinutes = CreateConVar("sm_adjuka_every_x_deaths_minutes", "10", "Should ad be played after a death where a certain amount of time has passed since the last ad (In Minutes), 0 = Disabled.", _, true, 0.0);
	g_ConVar_PeriodAmountMinutes = CreateConVar("sm_adjuka_ad_period", "0", "How often the periodic adverts should be played (In Minutes), 0 = Disabled.", _, true, 0.0);
	g_ConVar_SpectatorAmountMinutes = CreateConVar("sm_adjuka_ad_spectator_period", "5", "How often the spectator adverts should be played (In Minutes), 0 = Disabled.", _, true, 0.0);
	g_ConVar_OnPhase = CreateConVar("sm_adjuka_onphase", "0", "Display adverts during certain events such as winning and round start", _, true, 0.0, true, 1.0);
	g_ConVar_ExcludeAdmins = CreateConVar("sm_adjuka_excludeadmins", "0", "Don't display adverts to donators (Checks for generic admin flag)", _, true, 0.0, true, 1.0);
	g_ConVar_ExcludeDonators = CreateConVar("sm_adjuka_excludedonators", "0", "Don't display adverts to donators (Checks for reserved slot flag)", _, true, 0.0, true, 1.0);
	g_ConVar_KickNoMOTDEnabled = CreateConVar("sm_adjuka_kick_no_motd", "0", "Should players be kicked if they disabled motd, 0 = Disabled.", _, true, 0.0, true, 1.0);

	AutoExecConfig(true, "plugin.adjuka");

	// Get Engine Version
	g_EngineVersion = GetEngineVersion();

	//Event Hooks

	// Start
	HookEventEx("round_start", Event_OnPhase, EventHookMode_Post);
	HookEventEx("teamplay_round_start", Event_OnPhase, EventHookMode_Post);


	// Win
	HookEventEx("game_win", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("round_win", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("teamplay_win_panel", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("arena_win_panel", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("cs_win_panel_match", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("wave_complete", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("dod_win_panel", Event_OnPhase, EventHookMode_Pre);

	// End
	HookEventEx("game_end", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("tf_game_over", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("dod_game_over", Event_OnPhase, EventHookMode_Pre);
	HookEventEx("announce_phase_end", Event_OnPhase, EventHookMode_Pre);

	// Misc
	HookEvent("player_death", Event_OnClientDeath, EventHookMode_Post);
	HookEvent("player_team", Event_OnTeamSwitch, EventHookMode_Post);

	// Command Listeners
	AddCommandListener(CommandListener_CloseMOTDPage, "closed_htmlpage");

	// Commands
	RegAdminCmd("sm_showmotdad", Command_ShowAdvert, ADMFLAG_ROOT);

	// Late Load
	for (int i = 1; i <= MaxClients; i++)
	{
		if(IsValidClient(i))
		{
			OnClientPutInServer(i);
		}
	}

	CheckExtensions();
}

/* OnLibraryAdded()
** -------------------------------------------------------------------------- */
public void OnLibraryAdded(const char[] sLibraryName)
{
	CheckExtensions();
}

/* CheckExtensions()
** -------------------------------------------------------------------------- */
public void CheckExtensions()
{
	if (CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "SteamWorks_GetPublicIP") == FeatureStatus_Available)
	{
		g_bSteamWorks = true;
	}

	if (CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "Steam_GetPublicIP") == FeatureStatus_Available)
	{
		g_bSteamTools = true;
	}
}

/* OnMapStart()
** -------------------------------------------------------------------------- */
public void OnMapStart()
{
	ConVar ConVar_ServerPort = FindConVar("hostport");

	if(ConVar_ServerPort == null)
	{
		LogError("Could not retrieve server's port");
	}

	g_iHostPort = ConVar_ServerPort.IntValue;
	GetIP();

	GetGameFolderName(g_sGameDir, sizeof(g_sGameDir));

	ConVar ConVar_ServerHostName = FindConVar("hostname");

	if(ConVar_ServerHostName == null)
	{
		LogError("Could not retrieve server's name");
	}
}

/* Event_OnClientDeath()
** -------------------------------------------------------------------------- */
public Action Event_OnClientDeath(Event event, const char[] sName, bool dontBroadcast)
{
	int iClient = GetClientOfUserId(event.GetInt("userid"));

	if(!IsValidClient(iClient) || g_ConVar_XAmountDeaths.IntValue <= 0)
	{
		return Plugin_Continue;
	}

	if(g_EngineVersion == Engine_TF2 && (event.GetInt("death_flags") & TF_DEATHFLAG_DEADRINGER))
	{
		return Plugin_Continue;
	}

	if(g_iDeaths[iClient] < g_ConVar_XAmountDeaths.IntValue && (!g_ConVar_XAmountDeathMinutesEnabled.BoolValue || GetTime() - g_iTimeLastAd[iClient] < g_ConVar_XAmountDeathMinutes.IntValue * 60))
	{
		g_iDeaths[iClient]++;
	}
	else
	{
		g_iDeaths[iClient] = 0;
		CacheBusting(iClient);
	}

	return Plugin_Continue;
}

/* Event_OnTeamSwitch()
** -------------------------------------------------------------------------- */
public Action Event_OnTeamSwitch(Event event, const char[] sName, bool dontBroadcast)
{
	int iClient = GetClientOfUserId(event.GetInt("userid"));
	if(!IsValidClient(iClient))
	{
		return Plugin_Continue;
	}

	int iTeam = event.GetInt("team");
	if(iTeam == 1)
	{
		g_hSpectatorTimer[iClient] = CreateTimer(g_ConVar_SpectatorAmountMinutes.IntValue * 60.0, Timer_PlayAdvert, GetClientSerial(iClient), TIMER_REPEAT);
	}
	else
	{
		if(g_hSpectatorTimer[iClient] != INVALID_HANDLE)
		{
			delete g_hSpectatorTimer[iClient];
		}
	}

	return Plugin_Continue;
}

/* Event_OnPhase()
** -------------------------------------------------------------------------- */
public Action Event_OnPhase(Event event, char[] sEvent, bool bDontBroadcast)
{
	if(!g_ConVar_OnPhase.BoolValue)
	{
		return Plugin_Continue;
	}

	for(int iClient = 1; iClient <= MaxClients; iClient++)
	{
		CacheBusting(iClient);
	}

	return Plugin_Continue;
}

/* OnClientPutInServer()
** -------------------------------------------------------------------------- */
public void OnClientPutInServer(int iClient)
{
	// Set Defaults
	g_iDeaths[iClient] = 0;
	g_bHasMOTDEnabled[iClient] = true;
	g_bFirstJoin[iClient] = true;
	g_iTimeLastAd[iClient] = GetTime();

	if(!IsValidClient(iClient))
	{
		return;
	}

	if(g_ConVar_PeriodAmountMinutes.IntValue > 0)
	{
		g_hPeriodicTimer[iClient] = CreateTimer(g_ConVar_PeriodAmountMinutes.IntValue * 60.0, Timer_PlayAdvert, GetClientSerial(iClient), TIMER_REPEAT);
	}

	QueryClientConVar(iClient, "cl_disablehtmlmotd", Callback_OnCvarQuery);
}

/* Callback_OnCvarQuery()
** -------------------------------------------------------------------------- */
public void Callback_OnCvarQuery(QueryCookie cookie, int iClient, ConVarQueryResult conVarQueryResult, const char[] sCvarName, const char[] sCvarValue, int iQueryCount)
{
	if(conVarQueryResult != ConVarQuery_Okay)
	{
		return;
	}

	if(!IsValidClient(iClient))
	{
		return;
	}

	g_bHasMOTDEnabled[iClient] = StringToInt(sCvarValue) ? false : true;

	if(g_ConVar_KickNoMOTDEnabled.BoolValue && !g_bHasMOTDEnabled[iClient])
	{
		KickClient(iClient, "Type cl_disablehtmlmotd 0 into console!");
	}
}

/* Event_PlayerVGUIMenu()
** -------------------------------------------------------------------------- */
public Action Event_PlayerVGUIMenu(UserMsg iMessageId, Handle hBitBuffer, const int[] iPlayers, int iPlayersNum, bool bReliable, bool bInit)
{
	if(!IsValidClient(iPlayers[0]) || IsClientImmune(iPlayers[0]))
	{
		return Plugin_Continue;
	}

	char sCommunityID[512];
	g_ConVar_CommunityID.GetString(sCommunityID, sizeof(sCommunityID));
	if(StrEqual(sCommunityID, ""))
	{
		return Plugin_Continue;
	}

	char sBuffer1[64];
	char sBuffer2[256];

	BfReadString(hBitBuffer, sBuffer1, sizeof(sBuffer1));
	if (strcmp(sBuffer1, "info") != 0)
	{
		return Plugin_Continue;
	}

	if (BfReadByte(hBitBuffer) != 1)
	{
		return Plugin_Continue;
	}

	int iCount = BfReadByte(hBitBuffer);

	if (iCount == 0)
	{
		return Plugin_Continue;
	}

	KeyValues keyValues = new KeyValues("data");

	for (int i = 0; i < iCount; i++)
	{
		BfReadString(hBitBuffer, sBuffer1, sizeof(sBuffer1));
		BfReadString(hBitBuffer, sBuffer2, sizeof(sBuffer2));

		if (StrEqual(sBuffer1, "customsvr") || (StrEqual(sBuffer1, "msg") && !StrEqual(sBuffer2, "motd")))
		{
			delete keyValues;
			return Plugin_Continue;
		}

		keyValues.SetString(sBuffer1, sBuffer2);
	}

	DataPack dataPack = new DataPack();

	dataPack.WriteCell(GetClientUserId(iPlayers[0]));
	dataPack.WriteCell(view_as<int>(keyValues));

	CreateTimer(0.1, Timer_InitialAd, dataPack, TIMER_FLAG_NO_MAPCHANGE);

	return Plugin_Handled;
}

/* Timer_InitialAd()
** -------------------------------------------------------------------------- */
public Action Timer_InitialAd(Handle hTimer, DataPack dataPack)
{
	dataPack.Reset();

	int iClient = GetClientOfUserId(dataPack.ReadCell());
	KeyValues keyValues = view_as<KeyValues>(dataPack.ReadCell());

	delete dataPack;

	PlayAdvert(iClient, keyValues);

	return Plugin_Handled;
}

/* Timer_PlayAdvert()
** -------------------------------------------------------------------------- */
public Action Timer_PlayAdvert(Handle hTimer, int iClientSerial)
{
	int iClient = GetClientFromSerial(iClientSerial);

	if (!IsValidClient(iClient) && !IsClientImmune(iClient))
	{
		return Plugin_Handled;
	}

	CacheBusting(iClient);

	return Plugin_Continue;
}

/* CacheBusting()
** -------------------------------------------------------------------------- */
stock void CacheBusting(int iClient, bool bExcludeImmune = false)
{
	char sCommunityID[512];
	g_ConVar_CommunityID.GetString(sCommunityID, sizeof(sCommunityID));
	if(StrEqual(sCommunityID, ""))
	{
		return;
	}

	if(!bExcludeImmune && IsClientImmune(iClient))
	{
		return;
	}

	PlayCacheBuster(iClient);
	CreateTimer(1.0, Timer_AdAfterCacheBusting, GetClientSerial(iClient));
}

/* Timer_AdAfterCacheBusting()
** -------------------------------------------------------------------------- */
public Action Timer_AdAfterCacheBusting(Handle hTimer, int iClientSerial)
{
	int iClient = GetClientFromSerial(iClientSerial);

	if (!IsValidClient(iClient))
	{
		return Plugin_Stop;
	}

	PlayAdvert(iClient);

	return Plugin_Handled;
}

/* OnClientDisconnect()
** -------------------------------------------------------------------------- */
public void OnClientDisconnect(int iClient)
{
	g_iDeaths[iClient] = 0;

	if(g_hPeriodicTimer[iClient] != INVALID_HANDLE)
	{
		delete g_hPeriodicTimer[iClient];
	}

	if(g_hSpectatorTimer[iClient] != INVALID_HANDLE)
	{
		delete g_hSpectatorTimer[iClient];
	}
}

/* CommandListener_CloseMOTDPage()
** -------------------------------------------------------------------------- */
public Action CommandListener_CloseMOTDPage(int iClient, const char[] sCommand, int iArgs)
{
	if(!IsValidClient(iClient))
	{
		return Plugin_Continue;
	}

	if(g_bFirstJoin[iClient])
	{
		g_bFirstJoin[iClient] = false;
	}

	return Plugin_Continue;
}

/* Command_ShowAdvert()
** -------------------------------------------------------------------------- */
public Action Command_ShowAdvert(int iClient, int iArgs)
{
	if(!IsValidClient(iClient))
	{
		return Plugin_Handled;
	}

	char sCommunityID[512];
	g_ConVar_CommunityID.GetString(sCommunityID, sizeof(sCommunityID));
	if(StrEqual(sCommunityID, ""))
	{
		ReplyToCommand(iClient, "[AdJuKa] You have not specified a community ID in the plugin config");
		return Plugin_Handled;
	}

	CacheBusting(iClient, true);
	return Plugin_Handled;
}

/* PlayCacheBuster()
** -------------------------------------------------------------------------- */
stock void PlayCacheBuster(int iClient)
{
	if(!IsValidClient(iClient))
	{
		return;
	}

	char sURL[512];
	Format(sURL, sizeof(sURL), "https://google.com");

	KeyValues keyValues = new KeyValues("data");

	keyValues.SetNum("cmd", 5);
	keyValues.SetNum("type", MOTDPANEL_TYPE_URL);

	keyValues.SetString("title", "Google");
	keyValues.SetString("msg", sURL);

	ShowVGUIPanel(iClient, "info", keyValues, false);

	delete keyValues;
}

/* PlayAdvert()
** -------------------------------------------------------------------------- */
stock void PlayAdvert(int iClient, KeyValues keyValues = null)
{
	if(!IsValidClient(iClient))
	{
		return;
	}

	if(IsClientImmune(iClient))
	{
		return;
	}

	char sClientIP[64];
	if(!GetClientIP(iClient, sClientIP, sizeof(sClientIP)))
	{
		return;
	}

	if(StrEqual(g_sHostIP, "0.0.0.0"))
	{
		GetIP();
		return;
	}

	Call_StartForward(g_Forward_OnAdDisplayed);
	Call_PushCell(iClient);
	Call_Finish();

	g_iTimeLastAd[iClient] = GetTime();

	char sCommunityID[512];
	g_ConVar_CommunityID.GetString(sCommunityID, sizeof(sCommunityID));

	char sURL[256];
	Format(sURL, sizeof(sURL), "https://adjuka.com/motd/%s?version=%s&player=%s&server=%s:%i&game=%s", sCommunityID, PLUGIN_VERSION, sClientIP, g_sHostIP, g_iHostPort, g_sGameDir);
	if(keyValues == null)
	{
		keyValues = new KeyValues("data");
	}

	keyValues.SetNum("cmd", 5);
	keyValues.SetNum("type", MOTDPANEL_TYPE_URL);

	keyValues.SetString("title", "Adjuka MOTD Advert");
	keyValues.SetString("msg", sURL);
	keyValues.SetNum("customsvr", (!g_bFirstJoin[iClient] && g_EngineVersion == Engine_TF2) ? 1 : 0);

	ShowVGUIPanel(iClient, "info", keyValues, true);

	delete keyValues;
}

/* IsClientImmune()
** -------------------------------------------------------------------------- */
stock bool IsClientImmune(int iClient)
{
	if(!g_bHasMOTDEnabled[iClient])
	{
		return true;
	}

	if(g_ConVar_ExcludeDonators.BoolValue && GetAdminFlag(GetUserAdmin(iClient), Admin_Reservation))
	{
		return true;
	}

	if(g_ConVar_ExcludeAdmins.BoolValue && GetAdminFlag(GetUserAdmin(iClient), Admin_Generic))
	{
		return true;
	}

	return false;
}

/* IsValidClient()
** -------------------------------------------------------------------------- */
stock bool IsValidClient(int iClient, bool bNoBots = true)
{
	if (iClient <= 0 || iClient > MaxClients || !IsClientConnected(iClient) || (bNoBots && IsFakeClient(iClient)))
	{
		return false;
	}
	return IsClientInGame(iClient);
}

/* GetIP()
** -------------------------------------------------------------------------- */
stock void GetIP()
{
	char sServerAddress[64];
	g_ConVar_ServerIP.GetString(sServerAddress, sizeof(sServerAddress));
	char sServerAddressParts[2][64];

	if(!StrEqual(sServerAddress, "") && ExplodeString(sServerAddress, ":", sServerAddressParts, sizeof(sServerAddressParts), sizeof(sServerAddressParts[])) == 2)
	{
		Format(g_sHostIP, sizeof(g_sHostIP), "%s", sServerAddressParts[0]);
		g_iHostPort = StringToInt(sServerAddressParts[1]);
		return;
	}

	ConVar ConVar_ServerIP = FindConVar("hostip");
	if(ConVar_ServerIP != null)
	{
		int iIP = ConVar_ServerIP.IntValue;
		Format(g_sHostIP, sizeof(g_sHostIP), "%d.%d.%d.%d", iIP >>> 24 & 255, iIP >>> 16 & 255, iIP >>> 8 & 255, iIP & 255);

		if(!IsLocalIP(iIP))
		{
			return;
		}
	}

	if(g_bSteamTools)
	{
		Steam_GetPublicIP(g_iOctets);
		CreateIP(g_iOctets, g_sHostIP, sizeof(g_sHostIP));
	}
	else if(g_bSteamWorks)
	{
		SteamWorks_GetPublicIP(g_iOctets);
		CreateIP(g_iOctets, g_sHostIP, sizeof(g_sHostIP));
	}
	else
	{
		SetFailState("You need to install either the SteamTools or SteamWorks extension for this plugin to run.");
	}
}

/* CreateIP()
** -------------------------------------------------------------------------- */
stock void CreateIP(int iPieces[4], char[] sString, int iLength)
{
	Format(sString, iLength, "%i.%i.%i.%i", iPieces[0], iPieces[1], iPieces[2], iPieces[3]);
}

/* IsLocalIP()
** -------------------------------------------------------------------------- */
stock bool IsLocalIP(int iIP)
{
	if (iIP == 0 || 167772160 <= iIP <= 184549375 || 2886729728 <= iIP <= 2887778303 || 3232235520 <= iIP <= 3232301055)
	{
		return true;
	}

	return false;
}